﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class RobotinOhjaus : MonoBehaviour
{
    // Liikuttaaksemme robottia fysiikkamoottorin avulla, käytetään sen Rigidbody-komponenttia.
    // Tallennetaan viittaus komponenttiin tähän muuttujaan.
    Rigidbody rb;

    // Määrittää, millä nopeudella robotti liikkuu.
    // Tätä voi säätää inspektorista, koska tämä muuttuja on julkinen.
    public float liikkumisNopeus = 2.5f;

    // Määrittää, millä nopeudella robotti kääntyy.
    // Tätä voi säätää inspektorista, koska tämä muuttuja on julkinen.
    public float kääntymisNopeus = 200f;

    // Kääntääksemme robotin yläosaa hiiren suuntaan, on meillä oltava viittaus sen Transform-komponenttiin.
    // Aseta tämä inspektorissa.
    public Transform yläosa;

    // Julkinen LayerMask-tyyppinen muuttuja listaa kaikki valittavissa olevat layerit pudotusvalikkoon inspektorissa.
    // Rajoitetaan lähettämämme Raycast-säde osumaan vain tiettyyn layeriin (Maa).
    public LayerMask säteenVaikutus;

    // Start-funktio on Unity-pelimoottorin kutsuma ja se kutsutaan
    // kun tämä skripti ensimmäisen kerran tulee päälle, yleensä pelin alussa.
    void Start()
    {
        // Tallennetaan viittaus objektin Rigidbody-komponennttiin pelin alussa.
        rb = GetComponent<Rigidbody>();
    }

    // FixedUpdate-funktio on Unityn kutsuma ja se kutsutaan ennen Unityn fysiikkalaskelmia.
    // Tämän funktion kutsumataajuutta voi säätää Unity-projektin asetuksista Edit > Project Settings > Time välilehdestä.
    // FixedUpdate on tarkoitettu vain koodille, joka hyödyntää fysiikkaa.
    void FixedUpdate()
    {
        // Käytämme GetAxisRaw-funktiota pelaajan näppäinsyötteen saamiseen.
        // Tämä funktio vaatii parametrikseen haluttavan akselin nimen.
        // Unityssä on valmiiksi tehtyjä syöteakseleita, joita käytämme.
        // Funktio palauttaa arvon -1, 0 tai 1 riippuen painetuista näppäimistä.
        // Valmiit akselit ovat nähtävissä ja muokattavissa Unity-editorissa Edit > Project Settings > Input välilehdessä.
        // Siellä on Vertical-akselille asetettu positiiviseksi (1) näppäimeksi W ja negatiiviseksi (-1) S.
        // Horizontal-akselille on asetettu positiiviseksi (1) näppäimeksi D ja negatiiviseksi (-1) A.
        // Jos akselin kumpaakaan näppäintä ei paineta, palauttaa funktio arvon 0.

        // Otetaan talteen W ja S näppäinten syöte.
        float pystysuora = Input.GetAxisRaw("Vertical");

        // Siirtymä-vektori kuvastaa kuinka paljon siirrymme nykyisestä sijainnista, ja mihin suuntaan.
        // Voimme liikkua eteen- tai taaksepäin, jonka takia kerromme robotista eteenpäin osoittavan vektorin pystysuora-muuttujalla.
        // Tällöin saamme robotista eteenpäin osoittavan vektorin, jos W-näppäin on painettuna, taaksepäin osoittavan vektorin, jos S-näppäin on painettuna
        // tai nollavektorin, jos kumpaakaan näppäintä ei paineta. Nollavektori-tilanteessa robotti ei tule liikkumaan eteen tai taakse.
        // Kerromme tämän vektorin liikkumisNopeus-muuttujalla jolloin robotti liikkuu haluamallamme nopeudella.
        // Time-deltaTimellä kertomisen voi ajatella, että liikutamme tämän verran robottia sekunnissa.
        Vector3 siirtymä = transform.forward * pystysuora * liikkumisNopeus * Time.deltaTime;

        // Lopuksi asetetaan robotille sijainniksi nykyinen sijainti + siirtymä, eli uusi sijainti.
        rb.MovePosition(transform.position + siirtymä);

        // Otetaan talteen D ja A näppäinten syöte.
        float vaakasuora = Input.GetAxisRaw("Horizontal");

        // Objektin asentoa kuvataan Quaternion-tyypillä. Voimme kuitenkin muuttaa vektorimuodossa olevan asennon Quaternion muotoon
        // Quaternion.Euler-funktiota käyttämällä. Tämä funktio vaatii parametrikseen vektorin tai sen kolme komponenttia.
        // Emme halua robotin kääntyvän x- tai z-akselin suhteen, joten annetaan niiden arvoksi 0.
        // Kerrotaan vaakasuora-muuttuja, eli arvo -1, 0 tai 1 kääntymisnopeudella, jolloin saamme robotin kääntymään haluamallamme nopeudella.
        // Asetetaan se y-komponentin arvoksi.
        // Time-deltaTimellä taas halutaan robotin kääntyvän tämän verran sekunnissa.
        Quaternion kääntymä = Quaternion.Euler(0, vaakasuora * kääntymisNopeus * Time.deltaTime, 0);

        // Käännetään robotin asentoa kääntymän verran. Huomaa, että emme käytä plus-laskua, vaan kerromme nykyisen
        // asennon kääntymällä. Lisätietoa: https://docs.unity3d.com/ScriptReference/Quaternion-operator_multiply.html
        rb.MoveRotation(rb.rotation * kääntymä);
    }

    // Update-funktio on Unityn kutsuma ja se kutsutaan joka kerta kun näytölle piirretään kuva.
    // Jos peli pyörii 60 kuvaa sekunnissa, kutsutaan tämä funktio 60 kertaa sekunnin aikana.
    void Update()
    {
        // Jos peli on pysäytetty, älä suorita Update-funktiota loppuun.
        if (Time.timeScale == 0)
            return;

        // Saadaksemme robotin yläosan katsomaan hiiren osoittamaan suuntaan, on saatava selville mihin kohtaan hiiri osoittaa maailmassa.
        // Hyödynnämme Unityn raycast-toimintoa tämän selvittämiseksi. Raycastaus tarkoittaa näkymättömän säteen lähettämistä jostain
        // pisteestä johonkin suuntaan maailmassa ja sen osuessa johonkin collider-komponentilla varustettuun objektiin saamme tietoa siitä.
        // Tässä tapauksessa tarvitsemme sijainnin, jossa säde törmää johonkin maailmassa hiiren kohdalla.

        // Luodaan Ray-tyyppinen muuttuja, joka sisältää säteen lähetyssijainnin ja suunnan johon se lähetetään.
        // Camera-luokassa on valmis funktio, joka muuttaa ruudulla olevan pisteen maailman pisteeksi.
        // Tämä funktio on ScreenPointToRay ja se vaatii parametrikseen ruudun pisteen. Syötetään siihen Input-luokasta saatu hiiren sijainti.
        // Saamme viittaukseen maailman pääkameraan kirjoittamalla Camera.main. Tämä etsii Camera-komponentilla varustetun objektin, jonka tag on MainCamera.
        Ray säde = Camera.main.ScreenPointToRay(Input.mousePosition);

        // Tämä muuttuja tulee sisältämään säteen osumasta saadut tiedot vain jos säde osuu johonkin. Emme aseta tälle muuttujalle itse mitään arvoa.
        RaycastHit osuma;

        // Koska säde ei välttämättä osu mihinkään, on järkevää laittaa funktio, joka hoitaa säteen lähettämisen ehtolauseen sisään.
        // Tämä funktio on Physics-luokassa nimellä Raycast. Syötetään sille parametreiksi säde, osuma, maksimi etäisyys esim. 100 ja layer, johon sallimme säteen osuvan.
        // Varustamme RaycastHit-tyyppisen muuttujamme sanalla "out". Tämä tarkoittaa, että Raycast-funktion sisällä sille asetetaan arvo.
        if (Physics.Raycast(säde, out osuma, 100, säteenVaikutus))
        {
            // Saamme suunnan, johon yläosan pitäisi 'katsoa' laskemalla osuman ja robotin välisen vektorin.
            // Tämän saamme vähentämällä osuman sijainnista robotin sijainnin.
            Vector3 katsomisSuunta = osuma.point - transform.position;

            // Välttääksemme robotin katsomasta ylös tai alas pakotamme saamamme vektorin y-komponentin nollaksi.
            // Tämä siis tarkoittaa, että se tulee kääntymään vain y-akselin suhteen.
            katsomisSuunta.y = 0;

            // Lopuksi asetamme yläosan asennoksi Quaternion.LookRotation-funktiolla lasketun asennon, joka 'katsoo' vektorin suuntaan.
            yläosa.rotation = Quaternion.LookRotation(katsomisSuunta);
        }
    }
}
